#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#------------------------------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# IT WAS CREATED AND TESTED IN A WINDOWS ENVIRONMENT ONLY AND MAY NOT FUNCTION ON A MAC.
#------------------------------------------------------------------------------------------------
# THIS PLUGIN IS ANOTHER FIGMENT OF MY IMAGENATION AND IS NOT BASED ON OR COMPLY WITH ANYTHING! 
#------------------------------------------------------------------------------------------------
#    Name:	ComponentReplacement
#      By:	sdmitch
#   Usage:	Randomly replace the "Target" components with a "Source" component.
#	 	 Note:	All "Target" components MUST have the same name or prefix and all "Source" components MUST 
#						have the same name or prefix.  All "Target" components are randomly replaced by a "Source".
#						If some are selected, then only those are replaced.  The "Target" component can be anything
#						and can all be copies of a single component.  THE COMPONENT NAME, NOT THE COMPONENT
#						DEFINITION NAME IS USED FOR IDENTIFICATION OF THE "Source" AND "Target" COMPONENTS.  You
#						can access the component name through Entity Info or the Ruby Console by first selecting
#						the Source or Target components and using the appropriate code line below:
#						Sketchup.active_model.selection.each{|c| c.name='Source'}
#						Sketchup.active_model.selection.each{|c| c.name='Target'}
#    Date:	Jun 2011 Revised Feb 2013 for single source if "Yes" is choosen for random replacement.
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
# ------------------  MENU SETUP ---------------------- #
unless $sdm_tools_menu
	$sdm_tools_menu = UI.menu("Plugins").add_submenu("SDM Tools")
	$sdm_Edge_tools = $sdm_tools_menu.add_submenu("Edge Tool")
	$sdm_Face_tools = $sdm_tools_menu.add_submenu("Face Tool")
	$sdm_CorG_tools = $sdm_tools_menu.add_submenu("CorG Tool")
	$sdm_Misc_tools = $sdm_tools_menu.add_submenu("Misc Tool")
end
unless file_loaded?(__FILE__)
	$sdm_CorG_tools.add_item("Pilou Relative") { SDM::Pilou_Component_Replacement.main }
	file_loaded(__FILE__)
end
# ------------------------------------------------------ #
module SDM

	module Pilou_Component_Replacement

		def self.main
			mod=Sketchup.active_model
			ent=mod.active_entities
			sel=mod.selection
			@replace_random = "Yes" if !@replace_random;@replace_relative = "Yes" if !@replace_relative
			@source_name = "Source" if !@source_name;@target_name = "Target" if !@target_name
			prompts = ["Random Replacement?","Replace Relative?","Source Name/Prefix:","Target Name/Prefix:"]
			ans=UI.inputbox(prompts,[@replace_random,@replace_relative,@source_name,@target_name],["Yes|No","Yes|No"],"Component Replacement")
			return if !ans
			@replace_random,@replace_relative,@source_name,@target_name=ans;
			
			if @source_name==@target_name
				UI.messagebox "The source and target names must be different"
				return nil
			end
			
			# get the source component definitions
			source_components=[]
			for e in ent
				if e.is_a? Sketchup::ComponentInstance
					if e.name[0...@source_name.length]==@source_name
						source_components.push e
					end
				end
			end
			(UI.messagebox "No source components found";return) unless source_components[0]
			
			#get the target components definitions
			target_components=[]; sel = ent if sel.empty?
			for e in sel
				if e.is_a? Sketchup::ComponentInstance
					if e.name[0...@target_name.length]==@target_name
						target_components.push e
					end
				end
			end
			(UI.messagebox "No target components found";return) unless target_components[0]
			
			mod.start_operation "Component Replacement"
			@last=0; @replace_random="No" if source_components.length==1
			for tc in target_components
				tcwid=tc.bounds.width
				tchgt=tc.bounds.height
				tcdep=tc.bounds.depth
				if @replace_random == "Yes"
					begin
						i=rand(source_components.length)
					end until i != @last
					tc.definition=source_components[i].definition
					if @replace_relative == "Yes"
						scwid=source_components[i].bounds.width
						schgt=source_components[i].bounds.height
						scdep=source_components[i].bounds.depth
						xfac=tcwid/scwid * 1.0/tc.transformation.scaleX
						yfac=tchgt/schgt * 1.0/tc.transformation.scaleY
						zfac=tcdep/scdep * 1.0/tc.transformation.scaleZ
						trs=Geom::Transformation.scaling(tc.transformation.origin,xfac,yfac,zfac)
						tc.transform! trs
					end
					@last = i
				else
					tc.definition=source_components[@last].definition
					if @replace_relative == "Yes"
						scwid=source_components[@last].bounds.width
						schgt=source_components[@last].bounds.height
						scdep=source_components[@last].bounds.depth
						xfac=tcwid/scwid * 1.0/tc.transformation.scaleX
						yfac=tchgt/schgt * 1.0/tc.transformation.scaleY
						zfac=tcdep/scdep * 1.0/tc.transformation.scaleZ
						trs=Geom::Transformation.scaling(tc.transformation.origin,xfac,yfac,zfac)
						tc.transform! trs
					end
					@last += 1; @last = 0 if @last == source_components.length
				end
			end
			mod.commit_operation
		end
		
	end
	
	class Geom::Transformation
	### From transformation.extensions.rb (c) TIG 2009
	### From original ideas by TBD and others ? ### TIG 20091010
		def scaleX
			Math.sqrt(self.to_a[0]**2+self.to_a[1]**2+self.to_a[2]**2)
		end
		def scaleY
			Math.sqrt(self.to_a[4]**2+self.to_a[5]**2+self.to_a[6]**2)
		end
		def scaleZ
			Math.sqrt(self.to_a[8]**2+self.to_a[9]**2+self.to_a[10]**2)
		end
	end#class
	
end
